"""Public API to exporting."""

from typing import List, Dict, Callable, Optional
from functools import partial
from redbrick.common.context import RBContext
from redbrick.utils.pagination import PaginationIterator
from redbrick.utils.rb_label_utils import clean_rb_label, flat_rb_format


class Export:
    """Primary interface to handling export from a project."""

    def __init__(self, context: RBContext, org_id: str, project_id: str) -> None:
        """Construct Export object."""
        self.context = context
        self.org_id = org_id
        self.project_id = project_id

    def redbrick_format(self, only_ground_truth: bool = True,) -> List[Dict]:
        """Export data into redbrick format."""
        temp = self.context.export.get_datapoints_output
        if not only_ground_truth:
            raise NotImplementedError()

        my_iter = PaginationIterator(
            partial(temp, self.org_id, self.project_id, 50, True)
        )

        def _parse_entry(item: Dict) -> Dict:
            items_presigned = item["itemsPresigned"]
            items = item["items"]
            name = item["name"]
            dp_id = item["dpId"]
            created_by = item["labelData"]["createdByEmail"]
            labels = [clean_rb_label(label) for label in item["labelData"]["labels"]]

            return flat_rb_format(
                labels, items, items_presigned, name, dp_id, created_by
            )

        return [_parse_entry(val) for val in my_iter]



"""Logging functions."""
from typing import Union
from termcolor import colored


def print_info(text: str) -> None:
    """Logging general information."""
    print(colored("[INFO]: ", "blue"), text)


def print_warning(text: str) -> None:
    """Logging warnings."""
    print(colored("[WARNING]: ", "yellow"), text)


def print_error(text: Union[str, Exception]) -> None:
    """Logging errors."""
    print(colored("[ERROR]: ", "red"), text)



"""Handlers to access APIs for getting projects."""

from typing import List, Dict

from redbrick.common.client import RBClient
from redbrick.common.project import ProjectRepoInterface


class ProjectRepo(ProjectRepoInterface):
    """Class to manage interaction with project APIs."""

    def __init__(self, client: RBClient) -> None:
        """Construct ProjectRepo."""
        self.client = client

    def get_project_name(self, org_id: str, project_id: str) -> str:
        """
        Get project name.

        Raise an exception if project does not exist.
        """
        query = """
            query sdkGetProjectName($orgId: UUID!, $projectId: UUID!){
                project(orgId: $orgId, projectId: $projectId){
                    name
                }
            }
        """
        variables = {"orgId": org_id, "projectId": project_id}
        response = self.client.execute_query(query, variables)
        if response.get("project", {}).get("name"):
            name: str = response["project"]["name"]
            return name

        raise Exception("Project does not exist")

    def get_stages(self, org_id: str, project_id: str) -> List[Dict]:
        """Get stages."""
        query = """
            query sdkGetStages($orgId: UUID!, $projectId: UUID!){
                stages(orgId: $orgId, projectId: $projectId){
                    stageName
                    brickName
                }
            }
        """
        variables = {"orgId": org_id, "projectId": project_id}
        response: Dict[str, List[Dict]] = self.client.execute_query(query, variables)
        return response["stages"]



"""Interface for getting basic information about a project."""

from typing import Dict, List
from abc import ABC, abstractmethod


class ProjectRepoInterface(ABC):
    """Abstract interface to Project APIs."""

    @abstractmethod
    def get_project_name(self, org_id: str, project_id: str) -> str:
        """
        Get project name.

        Raise an exception if project does not exist.
        """

    @abstractmethod
    def get_stages(self, org_id: str, project_id: str) -> List[Dict]:
        """Get stages."""
